//
//                                                                         
// Module: Internals Example Source File                                   
//                                                                         
// Description: Declarations for the Internals Example Plugin              
//                                                                         
//                                                                         
// This source code module, and all information, data, and algorithms      
// associated with it, are part of CUBE technology (tm).                   
//                           PROPRIETARY                                   
// Copyright (c) 1996-2006 Image Space Incorporated.  All rights reserved. 
//                                                                         
//                                                                         
// Change history:                                                         
//   tag.2005.11.30: created                                               
//   jmm.2006.01.06: modified for public                                   
//                                                                         
//

#define _CRT_SECURE_NO_WARNINGS 1

//***************************************************************************//
#include "Example.hpp"          // corresponding header file
#include <math.h>               // for atan2, sqrt
#include <stdio.h>              // for sample output
//#include <string>
//#include <iostream>

#pragma comment(lib, "Ws2_32.lib")


	SOCKET s;
	SOCKADDR_IN brdcastaddr;



//***************************************************************************//

//***************************************************************************//
// plugin information
// Change this information to siut your needs. rFactor will use this information
// to ensure that it is using the most recent version
unsigned g_uPluginID          = 2;
char     g_szPluginName[]     = "ExampleInternalsPlugin - 2006.01.06";
unsigned g_uPluginVersion     = 002;
unsigned g_uPluginObjectCount = 1;
InternalsPluginInfo g_PluginInfo;
//***************************************************************************//

//***************************************************************************//
// interface to plugin information
extern "C" __declspec(dllexport)
const char* __cdecl GetPluginName() { return g_szPluginName; }

extern "C" __declspec(dllexport)
unsigned __cdecl GetPluginVersion() { return g_uPluginVersion; }

extern "C" __declspec(dllexport)
unsigned __cdecl GetPluginObjectCount() { return g_uPluginObjectCount; }

// get the plugin-info object used to create the plugin.
extern "C" __declspec(dllexport)
PluginObjectInfo* __cdecl GetPluginObjectInfo( const unsigned uIndex )
{
  switch(uIndex)
  {
    case 0:
      return  &g_PluginInfo;
    default:
      return 0;
  }
}
//***************************************************************************//

//***************************************************************************//
// InternalsPluginInfo class
InternalsPluginInfo::InternalsPluginInfo()
{
  // put together a name for this plugin
  sprintf( m_szFullName, "%s - %s", g_szPluginName, InternalsPluginInfo::GetName() );
}
//***************************************************************************//

//***************************************************************************//
const char*    InternalsPluginInfo::GetName()     const { return ExampleInternalsPlugin::GetName(); }
const char*    InternalsPluginInfo::GetFullName() const { return m_szFullName; }
// Change this to suit your needs
const char*    InternalsPluginInfo::GetDesc()     const { return "Example Internals Plugin"; }
const unsigned InternalsPluginInfo::GetType()     const { return ExampleInternalsPlugin::GetType(); }
const char*    InternalsPluginInfo::GetSubType()  const { return ExampleInternalsPlugin::GetSubType(); }
const unsigned InternalsPluginInfo::GetVersion()  const { return ExampleInternalsPlugin::GetVersion(); }
void*          InternalsPluginInfo::Create()      const { return new ExampleInternalsPlugin(); }
//***************************************************************************//

//***************************************************************************//
// InternalsPlugin class
const char ExampleInternalsPlugin::m_szName[] = "InternalsPlugin";
const char ExampleInternalsPlugin::m_szSubType[] = "Internals";
const unsigned ExampleInternalsPlugin::m_uID = 1;
const unsigned ExampleInternalsPlugin::m_uVersion = 1;
//***************************************************************************//

//***************************************************************************//
PluginObjectInfo *ExampleInternalsPlugin::GetInfo() { return &g_PluginInfo; }
//***************************************************************************//

//***************************************************************************//
void ExampleInternalsPlugin::Startup()
{
	int portno = 4123;
    WORD w = MAKEWORD(1,1);
    WSADATA wsadata;
    ::WSAStartup(w, &wsadata);
	
	s = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
    
    char opt = 1; 
    setsockopt(s, SOL_SOCKET, SO_BROADCAST, (char*)&opt, sizeof(char));
    memset(&brdcastaddr,0, sizeof(brdcastaddr));
    brdcastaddr.sin_family = AF_INET;
    brdcastaddr.sin_port = htons(portno);
    brdcastaddr.sin_addr.s_addr = INADDR_BROADCAST;
}
//***************************************************************************//

//***************************************************************************//
void ExampleInternalsPlugin::EnterRealtime()
{
}
//***************************************************************************//

//***************************************************************************//
void ExampleInternalsPlugin::UpdateTelemetry( const TelemInfo &info )
{
    char buffer [200];
	int len = sizeof(brdcastaddr);
	// Compute some auxiliary info based on the above
   TelemVect3 forwardVector = { -info.mOriX.z, -info.mOriY.z, -info.mOriZ.z };
   TelemVect3 upVector      = {  info.mOriX.y,  info.mOriY.y,  info.mOriZ.y };
   TelemVect3 leftVector    = {  info.mOriX.x,  info.mOriY.x,  info.mOriZ.x };

    // These are normalized vectors, and remember that our world Y coordinate is up.  So you can
    // determine the current pitch and roll (w.r.t. the world x-z plane) as follows:
    const float pitch = atan2f( forwardVector.y, sqrtf( ( forwardVector.x * forwardVector.x ) + ( forwardVector.z * forwardVector.z ) ) );
	const float yaw   = atan2f(info.mOriX.z, info.mOriZ.z); 
    const float  roll = atan2f( leftVector.y,    sqrtf( ( leftVector.x    * leftVector.x )    + ( leftVector.z    * leftVector.z ) ) );
    const float radsToDeg = 57.296f;
	const float speed = sqrtf(info.mLocalVel.x * info.mLocalVel.x + info.mLocalVel.y * info.mLocalVel.y + info.mLocalVel.z * info.mLocalVel.z) * 3.6f;
    
    //fprintf( fo, "%s", cpTemp);
    //mpConsole->Write(cpTemp);
	//sprintf(buffer, "%.3f, %.3f, %.3f, %.3f, %.3f, %.3f\r\n", info.mLocalRot.z, info.mLocalRot.x, info.mLocalAccel.y, info.mLocalRot.y, - info.mLocalAccel.x, - info.mLocalAccel.z);
	sprintf(buffer, "%.3f, %.3f, %.3f, %.3f, %.3f, %.3f, %.3f, %.3f, %d\r\n",  roll * radsToDeg, pitch * radsToDeg, info.mLocalAccel.y, - yaw * radsToDeg, - info.mLocalAccel.x, - info.mLocalAccel.z, speed,  info.mEngineRPM, info.mGear);
	//                                                          Roll,               Pitch,              HeaveAcc,           Yaw,                SwayAcc,            SurgeAcc
    
	int ret = sendto(s, buffer, strlen(buffer), 0, (sockaddr*)&brdcastaddr, len);
}
//***************************************************************************//

//***************************************************************************//
void ExampleInternalsPlugin::UpdateGraphics( const GraphicsInfo &info )
{

}
//***************************************************************************//

//***************************************************************************//
bool ExampleInternalsPlugin::CheckHWControl( const char * const controlName, float &fRetVal )
{
  return( false );
}
//***************************************************************************//

//***************************************************************************//
bool ExampleInternalsPlugin::ForceFeedback( float &forceValue )
{
 return( false );
}
//***************************************************************************//

//***************************************************************************//
void ExampleInternalsPlugin::Shutdown()
{
    ::closesocket(s);
}
//***************************************************************************//

